/*
  In setting the time we will display the current time with first the hours, then minutes, then seconds
  flashing once per second. The encoder turning increments or decrements the currently flashing item
  and encoder button causes the next item to become flashing. A push with seconds flashing returns.
*/
void set_time() {
  int h = hour, m = minute, s = second;
  
  char enc_pos = 0;
  boolean enc_pressed = 0;
  
  boolean flash = false;
  
  int hms_current = 0;
  int *hms[] = {
    &h,
    &m,
    &s
  };

  while (1) {
    for (int i = 0; i < 3; i++) {
      if (i == hms_current && flash) {
        sprintf(display_buf + (i * 3), "  :");
      }
      else {
        sprintf(display_buf + (i * 3), "%02d:", *hms[i]);
      }
    }
    
    vfd_set_string(display_buf, 0);

    if (wait_for_input(500, &enc_pos, &enc_pressed)) {
      if (enc_pressed) {
        hms_current++;
        if (hms_current > 2) {
          ds3231_set_time(h, m, s);
          break;
        }
      }
      else {
        *hms[hms_current] += enc_pos;
        if (hms_current == 0 && *hms[hms_current] > 23) {
          *hms[hms_current] = 0;
        }
        else if (hms_current == 0 && *hms[hms_current] < 0) {
          *hms[hms_current] = 23;
        }
        else if (hms_current == 1 && *hms[hms_current] > 59) {
          *hms[hms_current] = 0;
        }
        else if (hms_current == 1 && *hms[hms_current] < 0) {
          *hms[hms_current] = 59;
        }
        else if (hms_current == 2 && *hms[hms_current] > 59) {
          *hms[hms_current] = 0;
        }
        else if (hms_current == 2 && *hms[hms_current] < 0) {
          *hms[hms_current] = 59;
        }
      }
      flash = false;
    }
    else {
      flash = !flash;
    }
  }
}

void set_date() {
  int m = month, d = date, y = year;
  
  char enc_moved = 0;
  boolean enc_pressed = 0;

  do {
    m += enc_moved;
    if (m < 1) {
      m = 12;
    }
    else if (m > 12) {
      m = 1;
    }
    if (enc_pressed) {
      ds3231_set_date(y, m, d);
      break;
    }
    sprintf(display_buf, "Mo %s", month_abbrevs[m - 1]);
    vfd_set_string(display_buf, 0);
  } while (wait_for_input(MENU_TIMEOUT, &enc_moved, &enc_pressed));
  
  enc_moved = 0;
  enc_pressed = 0;
  do {
    d += enc_moved;
    if (d < 1) {
      d = 31;
    }
    else if (d > 31) {
      d = 1;
    }
    if (enc_pressed) {
      ds3231_set_date(y, m, d);
      break;
    }
    sprintf(display_buf, "Day %d", d);
    vfd_set_string(display_buf, 0);
  } while (wait_for_input(MENU_TIMEOUT, &enc_moved, &enc_pressed));
  
  enc_moved = 0;
  enc_pressed = 0;
  do {
    y += enc_moved;
    if (y < 0) {
      y = 99;
    }
    else if (y > 99) {
      y = 0;
    }
    if (enc_pressed) {
      ds3231_set_date(y, m, d);
      break;
    }
    sprintf(display_buf, "Yr 20%02d", y);
    vfd_set_string(display_buf, 0);
  } while (wait_for_input(MENU_TIMEOUT, &enc_moved, &enc_pressed));
}

void set_display_mode() {
  char *modes[] = {
    "hh mm ss",
    "hhmm  DD",
    "hhmm TTF",
    "MMM DD",
    "tttttttt",
    "hhmm  vv",
  };
  
  char enc_moved = 0;
  boolean enc_pressed = 0;
  
  char orig_display_mode = display_mode;

  do {
    display_mode += enc_moved;
    if (display_mode < 0) {
      display_mode = 5;
    }
    else if (display_mode > 5) {
      display_mode = 0;
    }
    if (enc_pressed) {
      save_config();
      orig_display_mode = display_mode;
      break;
    }
    vfd_set_string(modes[display_mode], 0);
  } while (wait_for_input(MENU_TIMEOUT, &enc_moved, &enc_pressed));
  
  display_mode = orig_display_mode;
}

void set_brightness() {
  char enc_moved = 0;
  boolean enc_pressed = 0;

  char orig_brightness = brightness;

  do {
    brightness += enc_moved;
    if (brightness < 0) {
      brightness = 0;
    }
    else if (brightness > 7) {
      brightness = 7;
    }
    
    vfd_data[0] = 0;
    for (int i = 1; i < 9; i++) {
      if (i == 1) {
        vfd_data[i] = VFD_SEG_F | VFD_SEG_E;
      }
      else if (i == 8) {
        vfd_data[i] = VFD_SEG_B | VFD_SEG_C;
      }
      else {
        vfd_data[i] = 0;
      }
      vfd_data[i] |= (brightness == i - 1 ? VFD_SEG_G : 0);
    }
    brightness_to_timer();
    
    if (enc_pressed) {
      save_config();
      orig_brightness = brightness;
      break;
    }
    
  } while (wait_for_input(MENU_TIMEOUT, &enc_moved, &enc_pressed));
  
  brightness = orig_brightness;
  brightness_to_timer();
  //voltage_target = 20.0 + ((brightness + 1) * 3.75);
}



void brightness_to_timer() {
  if (brightness <= 0) {
    OCR0A = 34; 
  } else if (brightness <= 1) {
    OCR0A = 42;
  } else if (brightness <= 2) {
    OCR0A = 46;
  } else if (brightness <= 3) {
    OCR0A = 54;
  } else if (brightness <= 4) {
    OCR0A = 60;
  } else if (brightness <= 5) {
    OCR0A = 70;
  } else if (brightness <= 6) {
    OCR0A = 80;
  } else if (brightness <= 7) {
    OCR0A = 90;
  } else {
    OCR0A = 30;
  }  

  //OCR0A = 34;   // Debug override
}

void set_bank_mode() {
  char *modes[] = {
    "Off",
    "On",
  };
  
  char enc_moved = 0;
  boolean enc_pressed = 0;
  
  char orig_bank_mode = bank_mode;

  do {
    bank_mode += enc_moved;
    if (bank_mode < 0) {
      bank_mode = 1;
    }
    else if (bank_mode > 1) {
      bank_mode = 0;
    }
    if (enc_pressed) {
      save_config();
      orig_bank_mode = bank_mode;
      break;
    }
    sprintf(display_buf, "Bank %s", modes[bank_mode]);
    vfd_set_string(display_buf, 0);
  } while (wait_for_input(MENU_TIMEOUT, &enc_moved, &enc_pressed));
  
  bank_mode = orig_bank_mode;
}

void main_menu() {
  /*
    TODO
    Change Display to Dsp Mode, add Time Fmt, add Date Fmt, make Bank a Dsp Mode option 
    along with Time, Date. Maybe. Kinda nice to have date/time formats too.
    Also add transition: Scroll, Switch
  */
  char *items[] = {
    "Set Time",
    "Set Date",
    "Display",
    "Britenes",
    "Bank",
    "Exit",
  };
  int items_length = 6;
  
  int index = 0;
  char enc_moved = 0;
  boolean enc_pressed = 0;

  do {
    index += enc_moved;
    if (index < 0) {
      index = items_length - 1;
    }
    else if (index > items_length - 1) {
      index = 0;
    }
    if (enc_pressed) {
      if (index == 0) {
        set_time();
        break;
      }
      else if (index == 1) {
        set_date();
        break;
      }
      else if (index == 2) {
        set_display_mode();
        break;
      }
      else if (index == 3) {
        set_brightness();
        break;
      }
      else if (index == 4) {
        set_bank_mode();
        break;
      }
      else {
        break;
      }
    }
    vfd_set_string(items[index], 0);
  } while (wait_for_input(MENU_TIMEOUT, &enc_moved, &enc_pressed));
}

