void updateRTC () {
    sei();
    ds3231_get_time((byte *) &hour, (byte *) &minute, (byte *) &second);
    ds3231_get_date((byte *) &year, (byte *) &month, (byte *) &date);
    temperature = ds3231_get_temp();
    millisecond = 0; 
}

void ds3231_setup() {
  Wire.begin();
  // set the control register
  ds3231_set_register(0x0e, 0b00000000);
}

byte dec_to_bcd(int dec) {
  return ((dec % 10) | ((dec / 10) << 4));
}

int bcd_to_dec(byte bcd) {
  return (((bcd >> 4) * 10) + (bcd & 0x0f));
}

void ds3231_set_register(byte address, byte value) {
  Wire.beginTransmission(DS3231_ADDR);
  Wire.send(address);
  Wire.send(value);
  Wire.endTransmission();
}

byte ds3231_get_register(byte address) {
  Wire.beginTransmission(DS3231_ADDR);
  Wire.send(address);
  Wire.endTransmission();
  Wire.requestFrom(DS3231_ADDR, 1);
  while (Wire.available() < 1);
  return Wire.receive();
}

void ds3231_set_time(byte hour, byte minute, byte second) {
  Wire.beginTransmission(DS3231_ADDR);
  Wire.send(0x00);
  Wire.send(dec_to_bcd(second));
  Wire.send(dec_to_bcd(minute));
  Wire.send(dec_to_bcd(hour));
  Wire.endTransmission();
}

void ds3231_get_time(byte *hour, byte *minute, byte *second) {
  Wire.beginTransmission(DS3231_ADDR);
  Wire.send(0x00);
  Wire.endTransmission();
  Wire.requestFrom(DS3231_ADDR, 3);
  while (Wire.available() < 3);
  byte s = Wire.receive();
  byte m = Wire.receive();
  byte h = Wire.receive();
  *second = bcd_to_dec(s);
  *minute = bcd_to_dec(m);
  *hour = bcd_to_dec(h);
}

void ds3231_set_date(byte year, byte month, byte date) {
  Wire.beginTransmission(DS3231_ADDR);
  Wire.send(0x04);
  Wire.send(dec_to_bcd(date));
  Wire.send(dec_to_bcd(month));
  Wire.send(dec_to_bcd(year));
  Wire.endTransmission();
}

void ds3231_get_date(byte *year, byte *month, byte *date) {
  Wire.beginTransmission(DS3231_ADDR);
  Wire.send(0x04);
  Wire.endTransmission();
  Wire.requestFrom(DS3231_ADDR, 3);
  while (Wire.available() < 3);
  byte d = Wire.receive();
  byte m = Wire.receive();
  byte y = Wire.receive();
  *date = bcd_to_dec(d);
  *month = bcd_to_dec(m & 0b00011111);
  *year = bcd_to_dec(y);
}

float ds3231_get_temp() {
  byte msb = ds3231_get_register(0x11);
  byte lsb = ds3231_get_register(0x12);
  float c = msb + ((lsb >> 6) * 0.25f);
  return c;
}

