/*
  Load the given VFD data into the SPI buffer and start the transfer.
  This sends the first byte of the data. The rest is sent from the
  SPI interrupt.
*/
static inline void vfd_send(unsigned long d) {
  spi_buffer = d << 8;
  spi_buffer_count = 3;
  
  SPDR = (spi_buffer >> 24);
  spi_buffer <<= 8;
  spi_buffer_count--;
}

/*
  Refresh the VFD by showing the next digit
*/

static inline void vfd_refresh() {
  /*if (vfd_digit_dwell_count--) {
    return;
  }
  vfd_digit_dwell_count = vfd_digit_dwell[8 - vfd_digit];  */
  char i;
  byte real_digit = vfd_digit;
  uint32_t t;
  uint32_t d = 0;
  byte fontdata_old = vfd_data[vfd_digit];
//  fontdata_old = 0xFF;
  d |= _BV(pgm_read_byte_near(digittable + real_digit));
  for (i=0; i<8; i++) {
    if (fontdata_old & _BV(i)) {
      t = 1;
      t <<=  pgm_read_byte_near(segmenttable + i);
      d |= t;
    }
  }  

  vfd_send(d);

  vfd_digit++;
  if (vfd_digit > 8) {
    vfd_digit = 0;
    
  }
}  
/*
  Map the given char to the correct VFD segment byte
  using the font table and some internals.
   a
 f   b  
   g  
 e   c   
   d
       h   
*/
byte vfd_map_char(char c) {
  if (c >= 'a' && c <= 'z') {
    return vfd_font[c - 'a' + 10]; 
  }
  else if (c >= 'A' && c <= 'Z') {
    return vfd_font[c - 'A' + 10];
  }
  else if (c >= '0' && c <= '9') {
    return vfd_font[c - '0']; 
  }
  else if (c == ' ') {
    return 0; 
  }
  else if (c == '-') {
    //return 0x02;
    return VFD_SEG_G;
  }
  else if (c == '.') {
    return VFD_SEG_H;
  }
  else if (c == '!') {
    return VFD_SEG_B | VFD_SEG_C | VFD_SEG_H;
  }
  else if (c == '?') {
    return VFD_SEG_A | VFD_SEG_B | VFD_SEG_E | VFD_SEG_G | VFD_SEG_H;
  }
  else if (c == ',') {
    return VFD_SEG_C;
  }
  else if (c == -1) {
    return VFD_SEG_G | VFD_SEG_H;
  }
  else {
    return 0; 
  }
}

void clear_display() {
  for (int i = 0; i < sizeof(display_buf); i ++) {
    display_buf[i] = 0;
  }
  vfd_data[0] = 0;
  vfd_set_string(display_buf, 0);    
}

/*
  Print the given string to the VFD
*/
void vfd_set_string(char *s, int offset) {
  int slen = strlen(s);
  
  Serial.print(s); 
  Serial.println();
 
  for (int i = offset; i < offset + 8; i++) {
    if (i < 0 || i > slen - 1) {
      vfd_data[i - offset + 1] = 0;
    }
    else {
      vfd_data[i - offset + 1] = vfd_map_char(s[i]);
    }
  }
}

void vfd_scroll_string(char *s, int del) {
  int slen = strlen(s);
  for (int i = -7; i < slen; i++) {
    vfd_set_string(s, i);
    delay2(del);
  }
}

